/**
 * Copyright 2005 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.nutch.mime;

// JDK imports
import java.util.ArrayList;


/**
 * Defines a Mime-Type.
 *
 * @author Jerome Charron - http://frutch.free.fr/
 */
final class MimeType {
    
    /** The Mime-Type name */
    private String name = null;

    /** The Mime-Type description */
    private String description = null;
    
    /** The Mime-Type associated extensions */
    private ArrayList extensions = null;
    
    /** The magic bytes associated to this Mime-Type */
    private ArrayList magics = null;
    
    private int minLength = 0;
    
    
    /** Creates a new instance of MimeType */
    MimeType(String name) {
        this(name, null);
    }

    /** Creates a new instance of MimeType */
    MimeType(String name, String description) {
        this.name = name;
        this.description = description;
        this.extensions = new ArrayList();
        this.magics = new ArrayList();
    }

    
    /**
     * Return the name of this mime-type.
     * @return the name of this mime-type.
     */
    String getName() {
        return name;
    }
    
    /**
     * Return the description of this mime-type.
     * @return the description of this mime-type.
     */
    String getDescription() {
        return description;
    }
    
    /**
     * Add a supported extension.
     * @param the extension to add to the list of extensions associated
     *        to this mime-type.
     */
    void addExtension(String ext) {
        extensions.add(ext);
    }

    /**
     * Return the extensions of this mime-type
     * @return the extensions associated to this mime-type.
     */
    public String[] getExtensions() {
        return (String[]) extensions.toArray(new String[extensions.size()]);
    }
    
    void addMagic(int offset, String type, String magic) {
        // Some preliminary checks...
        if ((magic == null) || (magic.length() < 1)) {
            return;
        }
        Magic m = new Magic(offset, type, magic);
        if (m != null) {
            magics.add(m);
            minLength = Math.max(minLength, m.size());
        }
    }
    
    int getMinLength() {
        return minLength;
    }
    
    boolean hasMagic() {
        return (magics.size() > 0);
    }
    
    boolean matches(byte[] data) {
        if (!hasMagic()) { return false; }
        
        Magic tested = null;
        for (int i=0; i<magics.size(); i++) {
            tested = (Magic) magics.get(i);
            if (tested.matches(data)) {
                return true;
            }
        }
        return false;
    }
    
    public String toString() {
        StringBuffer buf = new StringBuffer();
        buf.append("[").append(name)
           .append("(").append(description).append(") [");
        for (int i=0; i<extensions.size(); i++) {
            if (i>0) { buf.append(", "); }
            buf.append(extensions.get(i));
        }
        buf.append("] [");
        for (int i=0; i<magics.size(); i++) {
            if (i>0) { buf.append(", "); }
            buf.append(magics.get(i));
        }
        buf.append("]");
        return buf.toString();
    }

    private class Magic {
        
        private int offset;
        private byte[] magic = null;

        Magic(int offset, String type, String magic) {
            this.offset = offset;

            if ((type != null) && (type.equals("byte"))) {
                this.magic = readBytes(magic);
            } else {
                this.magic = magic.getBytes();
            }
        }
        
        int size() {
            return (offset + magic.length);
        }
        
        boolean matches(byte[] data) {
            if (data == null) { return false; }
            
            int idx = offset;
            if ((idx + magic.length) > data.length) {
                return false;
            }

            for (int i=0; i<magic.length; i++) {
                if (magic[i] != data[idx++]) {
                    return false;
                }
            }
            return true;            
        }
        
        private byte[] readBytes(String magic) {
            byte[] data = null;

            if ((magic.length() % 2) == 0) {
                String tmp = magic.toLowerCase();
                data = new byte[tmp.length() / 2];
                int byteValue = 0;
                for (int i=0; i<tmp.length(); i++) {
                    char c = tmp.charAt(i);
                    int number;
                    if (c >= '0' && c <= '9') {
                        number = c - '0';
                    } else if (c >= 'a' && c <= 'f') {
                        number = 10 + c - 'a';
                    } else {
                        throw new IllegalArgumentException();
                    }
                    if ((i % 2) == 0) {
                        byteValue = number * 16;
                    } else {
                        byteValue += number;
                        data[i/2] = (byte) byteValue;
                    }
                }
            }
            return data;
        }
        
        public String toString() {
            StringBuffer buf = new StringBuffer();
            buf.append("[").append(offset)
               .append("/").append(magic).append("]");
            return buf.toString();
        }
    }

}
